@@ -57,3 +57,10 @@ as.data.frame(df)
 fullgrid(df) = TRUE
 fullgrid(df) = FALSE
 as.data.frame(df)
+
+df = as.data.frame(df)
+set.seed(133331)
+df$xc = df$xc + rep(.001*rnorm(3), 3)[1:8]
+df.sp = SpatialPoints(df[c("xc", "yc")])
+df.grd = SpatialPixels(df.sp, tolerance = .01)
+df.grd[1:4,,tolerance=.01]
